# -*- coding: utf-8 -*-
# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Lesser General Public License version 3 (see the file LICENSE).

import simplejson
import urllib

from piston_mini_client import PistonSerializable

class BaseSerializer(object):
    def __init__(self, obj):
        self.obj = obj

    def serialize(self):
        raise NotImplementedError()


class JSONSerializer(BaseSerializer):
    """A serializer that renders JSON."""

    class PistonSerializableEncoder(simplejson.JSONEncoder):
        """A JSONEncoder that knows how to serialize PistonSerializables"""
        def default(self, o):
            if isinstance(o, PistonSerializable):
                return o._as_serializable()
            return simplejson.JSONEncoder.default(self, o)

    def serialize(self):
        return simplejson.dumps(self.obj, cls=self.PistonSerializableEncoder)


class FormSerializer(BaseSerializer):
    """A serializer that renders application/x-www-form-urlencoded content.

    NOTE this serializer doesn't support nested structures.  It should be
    initialized with a dictionary, sequence of pairs, or PistonSerializable.
    """
    def serialize(self):
        data = self.obj
        if isinstance(data, PistonSerializable):
            data = data._as_serializable()
        try:
            return urllib.urlencode(data)
        except TypeError:
            raise TypeError("Attempted to serialize invalid object %s" %
                self.obj)
serializers = {
    'application/json': JSONSerializer,
    'application/x-www-form-urlencoded': FormSerializer,
}
def get_serializer(content_type):
    return serializers.get(content_type)
