from unittest import TestCase

from piston_mini_client.serializers import JSONSerializer, FormSerializer
from piston_mini_client import PistonSerializable

class JSONSerializerTestCase(TestCase):
    def test_simple_serialize(self):
        serializer = JSONSerializer({'foo': 'bar'})
        self.assertEqual('{"foo": "bar"}', serializer.serialize())

    def test_piston_serializable_serialize(self):
        class MySerializable(PistonSerializable):
            _atts = ('foo',)

        myarg = MySerializable(foo='bar')
        serializer = JSONSerializer(myarg)
        self.assertEqual('{"foo": "bar"}', serializer.serialize())

    def test_serialize_fail(self):
        serializer = JSONSerializer({'foo': 3 + 1j})
        self.assertRaises(TypeError, serializer.serialize)

class FormSerializerTestCase(TestCase):
    def test_simple_serialize(self):
        serializer = FormSerializer({'foo': 'bar'})
        self.assertEqual('foo=bar', serializer.serialize())

    def test_piston_serializable_serialize(self):
        class MySerializable(PistonSerializable):
            _atts = ('foo', 'bar')

        myarg = MySerializable(foo='baz', bar=42)
        serializer = FormSerializer(myarg)
        self.assertEqual('foo=baz&bar=42', serializer.serialize())

    def test_serialize_nested(self):
        # Maybe we should flatly refuse to serialize nested structures?
        serializer = FormSerializer({'foo': {'a': 'b'}})
        self.assertEqual('foo=%7B%27a%27%3A+%27b%27%7D',
            serializer.serialize())
